@@ -1,5 +1,5 @@
 /*
- * Program:     $RCSfile: server.c,v $  $Revision: 4.9 $
+ * Program:     $RCSfile: server.c,v $  $Revision: 4.12 $
  *
  * Purpose:     UNIX application program that becomes a server for
  *              the Internet "youbin" service.
@@ -10,7 +10,7 @@
  *              S.Yamamoto  yamamoto@nuie.nagoya-u.ac.jp
  *
  * Date:        1993/07/24
- * Modified:    $Date: 1995/03/26 11:33:28 $
+ * Modified:    $Date: 1996/05/06 14:38:45 $
  *
  * Copyright:   K.Agusa and S.Yamamoto  1993 - 1995
  *
@@ -28,7 +28,7 @@
 
 #ifndef lint
 static char rcsid[] =
-    "$Id: server.c,v 4.9 1995/03/26 11:33:28 yamamoto Exp $";
+    "$Id: server.c,v 4.12 1996/05/06 14:38:45 yamamoto Exp yamamoto $";
 #endif
 
 #if defined(__STD__)
@@ -56,8 +56,9 @@ static char *usage[] = {
     "NAME",
     "    youbind - server for the Internet \"youbin\" service",
     "SYNOPSIS",
-    "    youbind [-d][-h][-t][-A port]",
+    "    youbind [-b header_list][-d][-h][-t][-A port]",
     "OPTIONS",
+    "    -b head  if more than Body,From:,To:,Subject:,cc:,Date:.",
     "    -d       Debug mode.",
     "    -h       Help: print out the usage.",
     "    -t       Trace: log the information of beginning and end of service.",
@@ -65,6 +66,7 @@ static char *usage[] = {
     "AUTHORS",
     "    K.Agusa     agusa@nuie.nagoya-u.ac.jp",
     "    S.Yamamoto  yamamoto@nuie.nagoya-u.ac.jp",
+    "Version 3.0 beta",
     NULL
 };
 
@@ -103,8 +105,8 @@ int         trace_mode = OFF;
 int         exitcode = EXIT_SUCCESS;
 
 /* The followings are only for debug. */
-static char *state_name[] = {
-    "ACKED", "IDLE", "WAIT", "TO1", "TO2", "NOT_USED"
+char *state_name[] = {
+    "ACKED", "IDLE", "WAIT", "TO1", "TO2", "NOT_USED", "PW1", "PW2", "NOT_USED"
 };
 
 /*
@@ -147,20 +149,13 @@ char    **argv;
     signal(SIGHUP, sig_hup);
     signal(SIGALRM, sig_alarm);
 
-    /*
-     * Set dummy time out interval. The time out never happen,
-     * because periodic() is called in every UNIT_TIME.
-     */
-    dummy.tv_sec = (long)(UNIT_TIME * 10);
-    dummy.tv_usec = 0L;
-
     /* Dive into main loop. Don't use setjmp() and longjmp(),
        because list maintenance routines are in critical section. */
     alarm(UNIT_TIME);
     while (1) {
         if (flag_hup == ON) {       /* First time call or catching SIGHUP. */ 
             send_Quit("hup");       /* Kill all clients. */
-            init_users();           /* Re-initialize users and states. */
+	    init_users();           /* Initialize users and states. */
             flag_hup = OFF;
         }
         if (flag_quit == ON) {      /* Catch SIGINT, SIGTERM or SIGQUIT. */ 
@@ -172,7 +167,15 @@ char    **argv;
             flag_timeout = OFF;
         }
         fdset_tmp = fdset;
-        if (select(max_fd, &fdset_tmp, 0, 0, &dummy) < 0) {
+        /*
+         * Set dummy time out interval. The time out never happen,
+         * because SIGALARM is sent in every UNIT_TIME.
+         * In some OS, especially Linux, contents of timeval structure
+         * become undefined. So they should be set in this loop.
+         */
+        dummy.tv_sec = (long)(UNIT_TIME * 10);
+        dummy.tv_usec = 0L;
+        if (SELECT(max_fd, &fdset_tmp, 0, 0, &dummy) < 0) {
             if (errno != EINTR) {
                 sys_error_log("select");
             }
@@ -192,8 +195,11 @@ init(argc, argv)
 int     argc;
 char    **argv;
 {
-    int   c;
+    int   c, headlen;
+    char  header_pattern[ 1000 ];
     
+    strcpy( header_pattern, BASIC_HEAD );
+    headlen = 1000 - strlen( header_pattern );
     if ((prog_name = strrchr(argv[0], '/')) != NULL) {
         prog_name++;
     } else {
@@ -203,11 +209,11 @@ char    **argv;
     strcpy(spool_path, MAIL_SPOOL);
     spool_file = spool_path + strlen(spool_path);
 
-    strcpy(NAK_buff, "NAK ");
+    strcpy(NAK_buff, "NACK ");
     NAK_reason = NAK_buff + strlen(NAK_buff);
 
     opterr = 0;
-    while ((c = getopt(argc, argv, "dhtA:S:")) != EOF) {
+    while ((c = getopt(argc, argv, "dhtb:A:S:")) != EOF) {
         switch (c) {
           case 'd':                         /* Debug mode. */
             debug_mode = ON;
@@ -219,7 +225,12 @@ char    **argv;
           case 't':                         /* Trace mode. */
             trace_mode = ON;
             break;
-          case 'A':                         /* Auxiliary port name. */
+          case 'b':                         /* biff headers */
+	    strcat( header_pattern, "," );
+	    strncat( header_pattern, optarg, headlen-1 );
+	    header_pattern[999] = 0;
+	    break;
+	  case 'A':                         /* Auxiliary port name. */
             aux_port_name = optarg;
             break;
           case 'S':                         /* Service port name. For debug. */
@@ -235,6 +246,7 @@ char    **argv;
         print_usage(usage);
         exit(EXIT_FAILURE);
     }
+    init_header_list( header_pattern );
 }
 
 void
@@ -244,14 +256,14 @@ int     fid;
     char            buff[MESS_LEN + 1];
     int             len, flen = sizeof(ca);
     
-    if ((len = recvfrom(fid, buff, MESS_LEN, 0, &ca, &flen)) < 0) {
+    if ((len = RECVFROM(fid, buff, MESS_LEN, 0, &ca, &flen)) < 0) {
         if (errno != EINTR) {
             warn_log("Error in receiving packet\n");
         }
         return;
     }
-    buff[len] = '\0';                       /* Zero termination. */
-    if (len < 2) {
+    buff[len] = '\0';                       /* Zero termination.          */
+    if (len < 4) {                          /* a@0\n is the shortest mess.*/
         debug_log("Too short length packet: %s\n", buff);
         return;
     }
@@ -263,20 +275,27 @@ int     fid;
                  ((cp != NULL) ? cp : "unknown"), ntohs(ca.sin_port));
         return;
     }
-    if (buff[1] != ' ') {
-        /* Trim the last newline.
-           Packet from /bin/mail has a ending newline. */
-        if (buff[len - 2] == '\n') {
-            buff[len - 2] = '\0';
-        } else if (buff[len - 1] == '\n') {
-            buff[len - 1] = '\0';
-        }
-        debug_log("Recv: from /bin/mail on port %s: \"%s\"\n",
-                  ((fid == sockid) ? service_name : aux_port_name), buff);
-        proc_mail(buff);
+
+#ifdef  VER2_BC
+    if( buff[1] == ' ' ){
+      ver2_proc( buff, len );
+      return;
+    }
+#endif    
+
+    if (buff[4] != ' ') {
+      /* Trim the last newline. Packet from /bin/mail has a ending newline. */
+      if (buff[len - 2] == '\n') {
+	buff[len - 2] = '\0';
+      } else if (buff[len - 1] == '\n') {
+	buff[len - 1] = '\0';
+      }
+      debug_log("Recv: from /bin/mail on port %s: \"%s\"\n",
+		((fid == sockid) ? service_name : aux_port_name), buff);
+      proc_mail(buff);
     } else {
-        debug_log("Recv: from client: \"%s\"\n", buff);
-        proc_client(buff, len);
+      debug_log("Recv: from client: \"%s\"\n", buff);
+      proc_client(buff, len);
     }
 }
 
@@ -284,33 +303,35 @@ void
 proc_mail(mess)
 char    *mess;
 {
-    /* 
-     * This routine sends a part of mail like biff.
-     * Add a flag "B" to Wakeup packet if you want.
-     */
-    char    *cp;
-    off_t   offset;
-    UserP   up;
-    StateP  sp;
+  /* 
+   * This routine sends a part of mail like biff.
+   * Add a flag "B" to Wakeup packet if you want.
+   */
+  char    *cp;
+  off_t   offset;
+  UserP   up;
+  StateP  sp;
     
-    if ((cp =  strchr(mess, '@')) == NULL) {
-        warn_log("Invalid packet from /bin/mail: %s\n", mess);
-        return;
-    }
-    
-    *cp = '\0';
-    if((up = find_user(mess)) == NULL) {    /* Check if user is registered. */
-        return;
-    }
-    offset = atol(cp + 1);                  /* Get offset address. */
-    check_spool(up);                        /* Update size and time. */
-    for (sp = up->stat; sp != NULL; sp = sp->next) {
-        if (sp->mode.biff) {
-            send_header(sp, offset);        /* Send header and so on. */
-        } else {
-            send_Status(sp);                /* Notify of modification. */
-        }
+  if ((cp =  strchr(mess, '@')) == NULL) {
+    warn_log("Invalid packet from /bin/mail: %s\n", mess);
+    return;
+  }
+  
+  *cp = '\0';
+  if((up = find_user(mess)) == NULL) {    /* Check if user is registered. */
+    return;
+  }
+  offset = atol(cp + 1);                  /* Get offset address. */
+  check_spool(up);                        /* Update size and time. */
+  for (sp = up->stat; sp != NULL; sp = sp->next) {
+    if( sp->state < NOT_USED ) {
+      if(sp->mode.head_list) {
+	send_header(sp, offset);        /* Send header and so on. */
+      } else {
+	send_Status(sp);                /* Notify of modification. */
+      }
     }
+  }
 }
 
 void
@@ -322,7 +343,7 @@ off_t   offset;
      * Some part of this function are copied from BSD source.
      */
     FILE    *fi;
-    int     len, line_cnt, char_cnt;
+    int     len, line_cnt, char_cnt, head, print_cont;
     int     flag_in_header;
     char    *line;
     char    buff[HEADER_LEN];
@@ -332,7 +353,12 @@ off_t   offset;
         sys_error_log("open");              /* Not created yet. */
         return;
     }
-    sprintf(buff, "S %d %d ", sp->parent->size, sp->parent->time);
+    sprintf(buff, "STAT %d %d ", sp->parent->size, sp->parent->time);
+#ifdef VER2_BC
+    if( sp->option.ver2_user ){
+      sprintf(buff, "S %d %d ", sp->parent->size, sp->parent->time);
+    }
+#endif  
     line = buff + strlen(buff);
     (void)fseek(fi, offset, 0);             /* Locate to proper position. */
 
@@ -344,17 +370,26 @@ off_t   offset;
     line_cnt = 7;
     char_cnt = 560;
     flag_in_header = ON;
+    head = sp->mode.head_list;
+    print_cont = OFF;
     while (fgets(line, sizeof(buff) - (line - buff), fi) != NULL) {
         if (flag_in_header == ON) {
             if (line[0] == '\n') {
+	        if(( head & 1 ) == 0 ) break;   /* Check if "Body" request */
                 flag_in_header = OFF;
                 continue;
             }
-            if (line[0] == ' ' || line[0] == '\t' ||
-                (strncmp(line, "From:", 5) && strncmp(line, "Subject:", 8))) {
-                continue;
-            }
-        }
+            if (line[0] == ' ' || line[0] == '\t' ){
+		if( print_cont == OFF ) continue;
+	    } else {
+		if( isPrintHead( line, head ) == FALSE ){
+		    print_cont = OFF;
+		    continue;
+		} else {
+		    print_cont = ON;
+		}
+	    }
+        } 
         if (line_cnt <= 0 || char_cnt <= 0) {
             strcpy(line, "...more...\n");
             break;
@@ -378,38 +413,18 @@ int   len;
 {
     StateP      id;                         /* User ID. */
     
-    switch (*buff) {
-      case 'W':                             /* Wakeup packet. */
-        do_Wakeup(buff + PREFIX_LEN, buff + len);
-        break;
-      case 'T':                             /* Thanks packet. */
-        if ((id = get_id(buff + PREFIX_LEN)) == NULL) { /* Invalid user. */
-            warn_log("Invalid Thanks packet: %s\n", buff);
-        } else {
-            debug_log("    Thanks packet: %s [%ld]: %s\n",
-                      id->parent->name, (long)id, state_name[id->state]);
-            id->state = ACKED;
-        }
-        break;
-      case 'U':                             /* Update packet. */
-        do_Update(buff + PREFIX_LEN);
-        break;
-      case 'Q':                             /* Quit packet. */
-        if ((id = get_id(buff + PREFIX_LEN)) == NULL) { /* Invalid user. */
-            warn_log("Invalid Quit packet: %s\n", buff);
-        } else {
-            char            buff_log[LOG_LEN];  /* Only for log. */
-
-            sprintf(buff_log, "Quit packet: %s [%ld] will be deleted\n",
-                    id->parent->name, (long)id);
-            debug_log("    %s", buff_log);
-            trace(buff_log);
-            del_state(id);
-        }
-        break;
-      default:
-        warn_log("Unknown packet: %s\n", buff);
-        break;
+    if( strncmp( buff, "THNQ ", 5 ) == 0 ){          /* Thank you packet.*/
+      do_Thank(buff + PREFIX_LEN);
+    } else if( strncmp( buff, "UPDT ", 5 ) == 0 ){   /* Update packet.   */
+      do_Update(buff + PREFIX_LEN );
+    } else if( strncmp( buff, "WAKE ", 5 ) == 0 ){  /* Wakeup packet.   */
+      do_Wakeup( buff + PREFIX_LEN );
+    } else if( strncmp( buff, "PASS ", 5 ) == 0 ){  /* Password packet. */
+      do_Passwd( buff+PREFIX_LEN );
+    } else if( strncmp( buff, "QUIT ", 5 ) == 0 ){   /* Quit packet.     */
+      do_Quit( buff + PREFIX_LEN );
+    } else {
+      warn_log("Unknown packet: %s\n", buff);
     }
 }
 
@@ -453,11 +468,13 @@ periodic()
     for (upp = &UserList; (up = upp->next) != NULL; upp = up) {
         for (dummy.next = up->stat, spp = &dummy; (sp = spp->next) != NULL;
              spp = sp) {
-            debug_log("    %s [%ld]: %s -> %s\n", up->name, (long)sp,
-                      state_name[sp->state], state_name[sp->state + 1]);
-            switch (sp->state) {
+	  debug_log("    %s [%ld]: %s -> %s\n", up->name, (long)sp,
+		    state_name[ sp->state ], state_name[ sp->state + 1 ] );
+	  switch (sp->state) {
               case ACKED:
-                sp->state = IDLE;
+	        if( sp->option.no_KeepAlive == FALSE ) {
+		  sp->state = IDLE;
+		}
                 break;
               case IDLE:
                 sp->state = WAIT;
@@ -471,16 +488,20 @@ periodic()
                 sp->state = TO2;
                 send_Status(sp);
                 break;
+	      case PW1:
+		sp->state = PW2;
+		break;
               default:                      /* Something wrong. */
                 warn_log("%s [%ld]: Unknown state: %d\n",
                          up->name, (long)sp, sp->state);
                 /* Don't break. */
+	      case PW2:                     /* Too late for PASS    */
               case TO2:                     /* TO2 and fatal error. */
                 spp->next = sp->next;
                 dispose_state(sp);
                 sp = spp;
                 break;
-            }
+	      }
         }
         if ((up->stat = dummy.next) == NULL) {
             upp->next = up->next;
@@ -543,7 +564,7 @@ SockAddr    *saddr;
     saddr->sin_family = hp->h_addrtype;
     saddr->sin_addr.s_addr = htonl(INADDR_ANY);
     saddr->sin_port = sp->s_port;
-    if (bind(id, saddr, sizeof(*saddr)) < 0) {
+    if (BIND(id, saddr, sizeof(*saddr)) < 0) {
         sys_error_log("bind");
         exit(EXIT_FAILURE);
     }
@@ -586,7 +607,7 @@ StateP      sp;
     debug_log("Send: %s [%ld]: \"%.*s\"\n",
               ((sp == CA_ADDR) ? "" : sp->parent->name), (long)sp,
               MAX_DEBUG_COLUMN, mess); 
-    if (sendto(sockid, mess, strlen(mess), 0, saddr, sizeof(*saddr)) < 0) {
+    if (SENDTO(sockid, mess, strlen(mess), 0, saddr, sizeof(*saddr)) < 0) {
         warn_log("Error in sending packet: %s\n", mess);
     }
 }
